import {
  Loader, Metadata, Router, I18n, dayjs as moment, IEventParam, IDictionary, DatasourceFactory, Config,
} from '@zeedhi/core';
import {
  DialogService, Select, Button, Form, IterableComponentRender, Text, LoadingService, Card, Tabs, Tab,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';

export class CardapioController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  private tipoCardapioSelected = '';

  private servicoCardapioSelected = '';

  private showCardapioDetail = false;

  get isVisibleDetailList() {
    return !this.showCardapioDetail;
  }

  public checkAccess() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
      if (!filial.value) {
        this.appController.unidadeLogadaIsValid();
      }
    } else {
      DialogService.show({
        name: 'needImport',
        type: 'error',
        title: I18n.translate('scm::ERROR'),
        text: I18n.translate('scm::NEED_IMPORT'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: {
              click: () => {
                DialogService.hide();
                Router.push('/importacao');
              },
            },
          },
        ],
      });
    }
  }

  public openFilter() {
    if (!localStorage.getItem('DATA_INICIAL') && !localStorage.getItem('DATA_FINAL')) {
      const btn = Metadata.getInstance<Button>('filterCardapioButton');
      btn.click();
    } else if (localStorage.getItem('CDFILIALLOGADA') !== 'null') {
      const dtIni = localStorage.getItem('DATA_INICIAL') || '';
      const dtFin = localStorage.getItem('DATA_FINAL') || '';
      this.loadCardapio(dtIni, dtFin, '');
    }
  }

  public loadCardapio(inicio: string, fim: string, search: string) {
    this.periodo.DTINICIAL = inicio;
    this.periodo.DTFINAL = fim;
    const cardapio = Metadata.getInstance<IterableComponentRender>('iterableCardsCardapio');
    cardapio.datasource.search = search;
    // af.datasource.order = [];
    cardapio.reload();
  }

  public clickFilterCardapio() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    const hasImportedData = localStorage.getItem('hasImportedData');
    if (filial.value && hasImportedData === 'true') {
      this.utilities.fillDatesFilter(bottomSheet);
      bottomSheet.isVisible = true;
      bottomSheet.isOpen = true;
    }
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public get setFilterTiposCardapio() {
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    return {
      CDFILCENTPME: filial.value?.CDFILCENTPME || '',
    };
  }

  public get setFilterServicosCardapio() {
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    return {
      CDFILCENTPME: filial.value?.CDFILCENTPME || '',
      CDFILIAL: filial.value?.CDFILIAL || '',
    };
  }

  public setPeriodoCardapio({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      if (form.value.PERIODO.length === 2) {
        let format = 'YYYY-MM-DD';
        if (form.value.PERIODO[1].charAt(2) === '/') format = 'DD/MM/YYYY';
        const dtini = moment(form.value.PERIODO[0], format);
        const dtfin = moment(form.value.PERIODO[1], format);
        if (dtini.isValid() && dtfin.isValid()) {
          if (dtini >= moment(localStorage.getItem('DATA_INICIAL'), 'DD/MM/YYYY')
            && dtfin <= moment(localStorage.getItem('DATA_FINAL'), 'DD/MM/YYYY')) {
            this.tipoCardapioSelected = form.value.CDTIPOCARD.CDTIPOCARD;
            this.servicoCardapioSelected = form.value.CDSERVFILI.CDSERVFILI;
            this.loadCardapio(dtini.format('DD/MM/YYYY'), dtfin.format('DD/MM/YYYY'), form.value.search);
            const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
            bottomSheet.isVisible = false;
            bottomSheet.isOpen = false;
          } else this.utilities.showAlert('errorPeriodFilter', 'DATE_NOT_IN_PERIOD', 'error');
        } else this.utilities.showAlert('errorDateFilter', 'INVALID_DATE', 'error');
      } else this.utilities.showAlert('errorFilter', 'FILTER_2_DATES', 'error');
    }
  }

  public get setFilterPratosCardapio() {
    if (localStorage.getItem('hasImportedData') === 'true'
      && this.tipoCardapioSelected !== '' && this.servicoCardapioSelected !== '') {
      const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
      return {
        CDFILCENTPME: filial.value?.CDFILCENTPME || '',
        CDTIPOCARD: this.tipoCardapioSelected || '',
        CDSERVFILI: this.servicoCardapioSelected || '',
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        DTCARDIAPME: this.utilities.getAllDatesInRange(this.periodo.DTINICIAL, this.periodo.DTFINAL),
      };
    }
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      CDFILCENTPME: '@',
      CDTIPOCARD: '@',
      CDSERVFILI: '@',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
  }

  public setFormattedDate({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[2] || '';
    component.text = `${this.utilities.formatDate(value)} - 
                    ${this.utilities.getDateFormat(value, 'dddd')}`;
  }

  public async setFormattedPlates({ component }: IEventParam<Text>) {
    LoadingService.show(' ');
    const key = component.name.split('-')[1] || '';
    const ds = DatasourceFactory.factory({
      uniqueKey: 'KEY',
      type: 'rest',
      route: '/pratosCardapios',
      filter: {
        KEY: key,
      },
    });
    await ds.get().then((result: IDictionary) => {
      const platesNames = result[0].PRATOS.map((el: any) => el.NMPRATO).join('<br>');
      component.text = platesNames;
      LoadingService.hide();
    });
  }

  public async clickCardDataCardapio({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    this.showCardapioDetail = true;
    const tabsDiasCardapio = Metadata.getInstance<Tabs>('tabsDiasCardapio');
    const diasRender = Metadata.getInstance<IterableComponentRender>('iterableCardsCardapio');
    if (tabsDiasCardapio.tabs.length > 0) {
      await Promise.all(tabsDiasCardapio.tabs.map(async (tab: IDictionary) => {
        await tab.children.map(async (child: IDictionary) => {
          await tab.removeChild(child.name);
        });
      })).then(() => {
        tabsDiasCardapio.tabs = [];
        tabsDiasCardapio.tabs = diasRender.datasource.data.map((row) => this.newTabs(row));
      });
    } else tabsDiasCardapio.tabs = diasRender.datasource.data.map((row) => this.newTabs(row));
    const index = component.name.split('-')[1];
    tabsDiasCardapio.activeTab = await tabsDiasCardapio.tabs.findIndex((tab) => tab.name === `${index}`);
    LoadingService.hide();
  }

  private newTabs(row: IDictionary) {
    return new Tab({
      name: row.KEY,
      tabTitle: `${this.utilities.formatDate(row.DTCARDIAPME)} - 
                    ${this.utilities.getDateFormat(row.DTCARDIAPME, 'dddd')}`,
      children: [
        {
          name: `iterableCardsPratos-${row.KEY}`,
          component: 'ZdIterableComponentRender',
          cssStyle: 'margin-bottom: 50px;',
          datasource: {
            uniqueKey: 'CDPRATO',
            type: 'memory',
            data: row.PRATOS,
          },
          componentMetadata: {
            name: 'cardPrato_<<row.KEY>>',
            component: 'ZdCard',
            cssClass: 'zd-my-2',
            cols: 12,
            hover: true,
            ripple: true,
            width: '100vh',
            children: [
              {
                name: 'title_<<row.KEY>>',
                component: 'ZdText',
                text: '<<row.NMPRATO>>',
                tag: 'h3',
                cssStyle: {
                  overflow: 'hidden',
                  'white-space': 'nowrap',
                  'text-overflow': 'ellipsis',
                },
              },
              {
                name: 'comensalPrato-<<row.KEY>>-<<row.QTCOMENSAL>>',
                component: 'ZdText',
                // text: `${I18n.translate('Comensal do Prato')}: <<row.QTCOMENSAL>>`,
                events: {
                  onMounted: '{{CardapioController.setValueQtde}}',
                },
              },
              {
                name: 'tipoPrato-<<row.KEY>>',
                component: 'ZdText',
                text: `${I18n.translate('Tipo de Prato')}: <<row.NMTIPOPRAT>>`,
              },
            ],
          },
        },
      ],
    });
  }

  public clickBackCardapio() {
    this.showCardapioDetail = false;
    const cardapios = Metadata.getInstance<IterableComponentRender>('iterableCardsCardapio');
    cardapios.reload();
  }

  public fillDatesFilter({ component }: IEventParam<Form>) {
    const dtini = moment(localStorage.getItem('DATA_INICIAL'), 'DD/MM/YYYY').format(Config.displayFormat);
    const dtfin = moment(localStorage.getItem('DATA_FINAL'), 'DD/MM/YYYY').format(Config.displayFormat);
    component.value.PERIODO = [dtini, dtfin];
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0];
    const mask = {
      decimalPlaces: 3,
    };
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${I18n.translate('Comensais do Prato')}: ${maskNumber}`;
  }
}
